/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Mateusz Moscicki <m.moscicki2@partner.samsung.com>
 */
#include <unistd.h>
#include <iostream>
#include <thread>
#include <cstring>

static constexpr size_t T_NUM = 8;
static constexpr size_t DEFAULT_COUNT = 1;
static constexpr size_t MB = 1024*1024;

// This function is used to check the correctness of gdb
// callstack, so id param may be useful in this case
static void MAGICNAME(int id) {
	while (true)
		sleep(1);
}

static bool alloc_chunk(size_t nr) {
	try {
		// For tests purposes we have to alloc some memory for nothing
		char *buff = new char[MB];
		std::memset(buff, nr % 256, MB);
	} catch (std::bad_alloc& er) {
		std::cout << "Memory allocation for " << nr << " chunk failed: " << er.what() << std::endl;
		return false;
	}

	return true;
}

static bool alloc_memory(size_t megabytes) {
	for (size_t i = 0; i < megabytes; i++) {
		if (!alloc_chunk(i))
			return false;
    }
	return true;
}

static void run(int id) {
	MAGICNAME(id);
}

int main(int argc, char *argv[]) {
	std::thread threads[T_NUM];

	for (size_t i = 0; i < T_NUM; i++) {
		threads[i] = std::thread(run, i);
		std::this_thread::sleep_for(std::chrono::microseconds(10));
	}

	size_t count = DEFAULT_COUNT;

	if (argc > 1)
		count = std::stoul(argv[1]);

	if (!alloc_memory(count))
		return EXIT_FAILURE;

	run(-1);

	return EXIT_SUCCESS;
}
